#!/usr/bin/env python

# Usage:
# replace_pos.py mecab_command JUMAN|IPA [mecab_options] < in.KNP > out.JDP
# replace_pos.py juman_command                           < in.KNP > out.JDP
#   replace gold POS in Kyoto Univ. Text Corpus w/ auto POS given by MeCab/JUMAN
#   * you may want to set mecab_optiuons to '-d your_target_dict'

import re, sys, subprocess

class Binfo:
    """ bunsetsu infomation """
    def __init__ (self, offset, head_id, ptype):
        self.offset, self.head_id, self.ptype = offset, head_id, ptype;
        self.morphemes = []
    def header (self): return "%d%s" % (self.head_id, self.ptype)
    def morph  (self): return '\n'.join (m for m in self.morphemes)

if len (sys.argv) < 2:
    sys.exit ("""Usage:
  python %s mecab_command JUMAN|IPA [mecab_options] < in.KNP > out.JDP
  python %s juman_command                           < in.KNP > out.JDP"""
              % (sys.argv[0], sys.argv[0]))

tagger = "JUMAN" if len (sys.argv) == 2 else "MeCab"
posset = "JUMAN"
if tagger == "JUMAN":
    sys.argv.append ("-b")
else:
    posset = sys.argv.pop (2)
    if posset != "JUMAN" and posset != "IPA":
        sys.exit ("Unknown POS set: %s\n" % posset)

t = subprocess.Popen (sys.argv[1:], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
header = ''
sent   = ''
binfo  = []
morphs = []
stat   = { 'success': 0, 'failed': 0 }
for line in sys.stdin:
    if line[0] == '#':
        header = line
        sent   = ''
        binfo[:] = []
    elif line[0] == 'E': # EOS
        footer = line
        binfo.append (Binfo (len (sent), -1, 'D')) # dummy
        offset = 0
        i = 0
        tagger_in  = t.stdin
        tagger_out = t.stdout
        tagger_in.write (sent + "\n")
        morphs = []
        while True:
            morphs.append (tagger_out.readline ()[:-1])
            if morphs[-1] == "EOS":
                break
        for morph in morphs:
            surface = ""
            feature = ""
            m = re.search ("^(\S+)[\s|\t](.+)$", morph)
            if m:
                surface = m.group (1)
                feature = m.group (2)
            # unknown words lacks fields related to surface form
            # modify here to be consistent with the MeCab output
            if binfo[i].offset == offset: # correct bunsetsu chunk
                i += 1
            elif i + 1 < len (binfo):
                if binfo[i + 1].offset == offset and binfo[i - 1].head_id == i:
                    # assuming compounds; you may want to add more conditions
                    binfo[i-1].head_id = binfo[i].head_id
                    del (binfo[i])
                    for bi in filter (lambda bj: bj.head_id >= i, binfo):
                        bi.head_id -= 1
                    i += 1
                elif binfo[i + 1].offset < offset: # mismatched
                    stat['failed'] += 1
                    break
            if surface:
                if posset == "IPA" and len (feature.split (',')) != 9:
                    morph += ",*,*" # complement empty info for unknown words
                binfo[i-1].morphemes.append (morph)
                offset += len (surface)
        else:
            stat['success'] += 1
            print header,
            for j in range (len (binfo) - 1):
                print "* %d %s" % (j, binfo[j].header ())
                print binfo[j].morph ()
            print footer,
    elif line[0] == '*':
        head_type = line[:-1].split(' ')[2]
        binfo.append (Binfo (len (sent), int (head_type[:-1]), head_type[-1]))
    else:
        sent += line[:-1].split(' ')[0]

sys.stderr.write ("(success, failed) = (%d, %d)\n"
                  % (stat['success'], stat['failed']))
